#from Interface import ZContact
from AccessControl import ClassSecurityInfo, getSecurityManager
from Products.ZLDAPMethods.LM import LDAPFilter, LDAPConnectionIDs
from Acquisition import Implicit
from Globals import Persistent
from Globals import DTMLFile, HTMLFile, MessageDialog
from AccessControl.Role import RoleManager
from OFS.SimpleItem import Item

# A list of the attributes that updateEntry() and addEntry() use
mod_attrs = ('cn','sn','mail','telephonenumber','o','postaladdress','facsimiletelephonenumber')
# A list of attributes that the search uses
search_attrs = ('o','cn','mail')
# Apart from postaladdress and cn, what attributes to show in search results
result_attrs = (['Organisaatio','o'],
                ['Puhelinnumero','telephonenumber'],
                ['Faksinumero','facsimiletelephonenumber'])

objAddForm = HTMLFile('dtml/objAddForm', globals())

def objAddFunction(dispatcher, id, connection_id, basedn, parentdn, REQUEST=None):
    """ Create a new ZContact object and add it """
    p=ZContactProduct(id, connection_id, basedn, parentdn)
    dispatcher.Destination()._setObject(id, p)

    if REQUEST is not None:
        u = REQUEST['URL1']
        u += '/manage_workspace'
        REQUEST.RESPONSE.redirect(u)
    return None

class ZContactProduct(Implicit, Persistent, RoleManager, Item):

    meta_type = 'ZContact'
    security = ClassSecurityInfo()

    manage_options = (
        {'label' : 'Properties', 'action':'manage_main'},
        {'label' : 'Security', 'action':'manage_access'},
        {'label' : 'Search contacts', 'action':'search'},
        {'label' : 'Add contacts', 'action':'add'}
    )

    def __init__(self, id, connection_id, basedn, parentdn):
        self.id = id
        self._set_properties(connection_id, basedn, parentdn)
        self._setupLDAPFilters()

    def _set_properties(self, connection_id, basedn, parentdn):
        """ Sets the class attributes connection_id and basedn """
        if connection_id:
            self.connection_id = connection_id
        else:
            raise AttributeError, 'empty connection_id?'
        if basedn:
            self.basedn = basedn
        else:
            raise AttributeError, 'empty basedn?'
        if parentdn:
            self.parentdn = parentdn
        else:
            raise AttributeError, 'empty parentdn?'


    ##
    # DTML methods

    # These will be published
    security.declareProtected('View', 'search')
    search = DTMLFile('dtml/search', globals())
    security.declareProtected('Modify contact information', 'add', 'edit')
    add = DTMLFile('dtml/add', globals())
    edit = DTMLFile('dtml/edit', globals())
    security.declareProtected('View management screens', 'manage_main')
    manage_main = DTMLFile('dtml/manage_propertiesForm', globals())
    # private
    security.declarePrivate('displaySearch', 'menu')
    displaySearch = DTMLFile('dtml/displaySearch', globals())
    menu = DTMLFile('dtml/menu', globals())


    ##
    # Published class methods

    security.declarePublic('index_html')
    def index_html(self, REQUEST=None):
        """
        If accessing the object as such, let's redirect to the searching form.
        """
        if REQUEST is not None:
            u = REQUEST['URL1']
            u += '/search'
            REQUEST.RESPONSE.redirect(u)
        return None

    security.declareProtected('Manage ZContacts', 'manage_properties')
    def manage_properties(self, connection_id, basedn, parentdn, REQUEST = None):
        """
        Does the necessary modifications when the basedn or connection ID
        is changed
        """
        self._set_properties(connection_id, basedn, parentdn)
        self._setupLDAPFilters()
        if REQUEST is not None:
            return MessageDialog(
                title='Properties updated',
                message='The properties of %s were updated.' % self.id,
                action='manage_main')


    ##
    # Methods called by the DTML methods

    security.declarePrivate('updateEntry')
    def updateEntry(self, REQUEST):
        """
        Update the entry in the database according to arguments passed via
        REQUEST

        Returns a tuple of the message telling whether the update was
        succesful or not, and the new dn.
        """
        (message, entryAttrs) = self._checkAttributesFromREQUEST(REQUEST)
        if not entryAttrs:
            return (message, REQUEST['dn'])

        entry = self.getEntryByDN(REQUEST = REQUEST)
        # We have to change the RDN if the cn-attribute changes
        newdn = entry.rename('cn='+entryAttrs['cn'])
        entry.setAll(entryAttrs)

        # Push the entry to REQUEST, so when getEntryByDN will be called
        # again it can return it straight away. This avoids some problems.
        REQUEST.set('targetEntry', entry)
        return ('Modified!', newdn)

    security.declarePrivate('addEntry')
    def addEntry(self, REQUEST):
        """
        Add an entry in the database according to arguments passed via
        REQUEST

        Returns a string indicating failure or success
        """
        (message, entryAttrs) = self._checkAttributesFromREQUEST(REQUEST)
        if not entryAttrs:
            return message

        # The new entry will be added directly under the root entry
        parentEntry = self.getEntryByDN({'dn' : self.parentdn})
        parentEntry.addSubentry('cn='+entryAttrs['cn'], entryAttrs)
        return "Added!"


    security.declarePrivate('getEntryByDN')
    def getEntryByDN(self, REQUEST):
        """
        Return the entry object that has the DN of REQUEST['dn']

        This had to be implemented in an eccentric way, that requires
        every entry to have an attribute identical to it's RDN, because the
        base used by LDAP filter objects without constructing them again.
        """
        if REQUEST.has_key('targetEntry'):
            return REQUEST['targetEntry']

        rdn = self._getRDNFromDN(REQUEST['dn'])
        entries = self.getEntryLDAPFilter(clause = rdn)

        for entry in entries:
            if entry.dn == REQUEST['dn']: return entry

        raise Exception, 'Entry not found? ' + REQUEST['dn'] + ' : ' + rdn

    security.declarePrivate('deleteEntry')
    def deleteEntry(self, REQUEST):

        if not REQUEST.has_key('dn'): raise Exception, 'No DN specified?'

        targetEntry = self.getEntryByDN(REQUEST)
        targetEntry.deleteSubentry(targetEntry)

    security.declarePrivate('decodePostaladdress')
    def decodePostaladdress(self, postaladdress):
        """
        Changes $s to newlines, used to decode the postaladdress before
        putting showing it in a form.
        """
        string = str(postaladdress)
        return string.replace('$', '\n')

    security.declarePrivate('canModify')
    def canModify(self, REQUEST):
        """
        Checks if the current user has the permission to modify
        contacts
        """
        return getSecurityManager().checkPermission('Modify contact information', self)

    security.declarePrivate('UTF8ToLatin')
    def UTF8ToLatin(self, string):
        return unicode(string, 'utf-8').encode('latin-1')

    security.declarePrivate('searchLDAPFilter')
    def searchLDAPFilter(self, REQUEST):
        """
        Convert latin to in the search parameters UTF and do the search
        itself.
        """
        params = {}
        for attr in search_attrs:
            if REQUEST.has_key(attr):
                params[attr] = self._latinToUTF8(REQUEST[attr])

        return self._searchLDAPFilter(params)

    security.declarePrivate('buildResultSequence')
    def buildResultSequence(self, dict):
        """
        Builds a sequence of sequence that we can use directly with dtml-in
        based on the entry's attributes passed in a dict.
        """
        sequence = []
        for attr in result_attrs:
            if dict.has_key(attr[1]) and dict[attr[1]]:
                sequence.append([attr[0], dict[attr[1]]])
        return sequence


    ##
    # "Internal" functions (not used by anything else than the python
    # methods of this class):

    def _setupLDAPFilters(self):
        """ Construct the needed LDAP filter objects """
        # The filter used by the search form
        self._searchLDAPFilter = LDAPFilter('searchFilter', '', self.connection_id,
                                          'SUBTREE', self.basedn,
                                          'o cn mail',
                                          '(&<dtml-if mail>(mail=*<dtml-var mail>*)</dtml-if>\
                                           <dtml-if o>(o=*<dtml-var o>*)</dtml-if> \
                                           <dtml-if cn>(cn=*<dtml-var cn>*)</dtml-if> \
                                           (!(objectclass=organization)))');

        # The search filter used to fetch single entries
        self.getEntryLDAPFilter = LDAPFilter('entryFilter', '', self.connection_id,
                                             'SUBTREE', self.basedn,
                                             'clause', '<dtml-var clause>');

    def _getRDNFromDN(self, dn):
        """ Extracts the RDN from a DN """
        offset = dn.find(',')
        return offset > -1 and dn[:offset] or dn

    def _checkAttributesFromREQUEST(self, REQUEST):
        """
        Tries to construct a dictionary of attributes from the REQUEST
        object.

        Does some checks and modifications and returns a tuple of a message
        indicating the possible failure and the dictionary on success.
        """
        # cn is the only mandatory attribute.
        if not REQUEST.has_key('cn') or not REQUEST['cn']:
            return ('Define a name!', None)

        # Some modifications to the REQUEST..
        REQUEST['cn'] = self._removeTrailingWhitespace(str(REQUEST['cn']))
        # We'll guess that the last word of the name is the surname
        names = str(REQUEST['cn']).split()
        if(len(names) > 1):
            REQUEST['sn'] = " ".join(names[-1:])
        if REQUEST.has_key('postaladdress'):
            REQUEST['postaladdress'] = \
              self._encodePostaladdress(postaladdress = REQUEST['postaladdress'])

        entryAttrs = {}
        for attr in mod_attrs:
            if REQUEST.has_key(attr) and REQUEST[attr] != '':
                entryAttrs[attr] = self._latinToUTF8(REQUEST[attr])

        return (None, entryAttrs)

    def _encodePostaladdress(self, postaladdress):
        """
        Changes newlines to $, used to encode the postaladdress before
        putting it into the database.
        """
        string = str(postaladdress)
        return string.replace('\n', '$')

    def _removeTrailingWhitespace(self, string):
        """ Removes tabs and spaces from the end of the string """
        while string[-1:] == ' ' or string[-1:] == '\t':
            string = string.rstrip()
        return string

    def _latinToUTF8(self, string):
        return unicode(string, 'latin-1').encode('utf-8')
